{% load listable %}
<table id="{{table_id}}" class="table">
    <thead>
        <tr>
            {% for field, header in fields %}<th data-field="{{field }}">{{header}}</th>{% endfor %}
        </tr>
        <tr>
          {% for field, header in fields %}<th data-field="{{field }}">{{header}}</th>{% endfor %}
        </tr>
    </thead>
    <tbody>
        {% for row in rows %}
            <tr>
                {% for td in row %}
                    <td>{{td}}</td>
                {% endfor %}
            </tr>
        {% endfor %}
    </tbody>
    <tfoot>
        <tr>
          {% for field, header in fields %}<th data-field="{{field }}">{{header}}</th>{% endfor %}
        </tr>
    </tfoot>

</table>
